--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  autosave.pl
--
--**************************************************************************

module AutoSave is
use public RegEvent                 -- Register/UnRegister Event library
use Workfile
#include       "EventErr"          -- Event Handler special error code defintions

#define  n   50                     -- SaveFile saves the worksheet every
                                   -- nth time it's called
integer        pollCount = 0               -- Initialize pollCount to 0

----------------------------------------------------------------------------
--
-- Procedure:     SaveFile
-- Description:   Increment pollCount until it reaches n.  When it's n,
--                set it back to 0 and save the current file.
--
----------------------------------------------------------------------------
procedure SaveFile   (EventTrigger   theTrigger,
                      MiscGroup      theEvent,
                      integer        theError)
          returns    (EventReturn    theReturn,
                      integer        errorCode,
                      string         errorString)
is
       -- Initialize error return values to NoError or NULL
       errorCode = NoError
       errorString = ""

   theReturn = EventContinue
   pollCount = pollCount + 1

   if (pollCount = n) then
      pollCount = 0
      Workfile [] w
      Save (w, ReplacementFile)     -- Replace the file on disk if it exists
      except on FileNotFound do     -- If not,
                Save (w, NewFile)   -- Save it as a new file
                except on others do    -- Ignore Save errors
                end except
             on others do           -- Ignore Save errors
      end except
   end if

   return
end procedure SaveFile



----------------------------------------------------------------------------
--
-- Procedure:     Initialize
-- Description:   Register SaveFile to be called in the Poll event.
--
----------------------------------------------------------------------------
procedure Initialize is
   EventHandle  SaveFileHandle
   SaveFileHandle = Register (MiscGroupHdlr (SaveFile), Poll, BeforeEvent)
end procedure Initialize


end module AutoSave
