--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  commport.pl
--
--**************************************************************************


module CommPort in CommPort is Primitive

--
-- Declare the abstract data type CommPort, the attributes and procedures
-- of which will be created as primitive (C) functions
--
abstract type CommPort


--
-- Declare enumerated type variables used by attributes
--
type PortType is oneof (COM1, COM2, COM3, COM4)

type ParityType is oneof (EVEN, ODD, NONE)

type DataBitType is oneof (SEVEN, EIGHT)

type StopBitType is oneof (ONE, TWO)


------------------------------------------------------------------
--
-- Declare the attributes and procedures implemented in C
--
-- You can declare overloaded versions of all of the procedures and
-- attributes except for the operator functions (of which EQ is the
-- example shown here).
--
--  Note:   These items must be in the same ORDER they appear in the
--          bind routine LPL callable exports section.
--
------------------------------------------------------------------

--
-- Constructor:
--
-- 1) Constructors always have the same name as the abstract type they are
--      implemented for.
-- 2) Constructors never take an instance of the abstract type as an input.
-- 3) Constructors only return an instance of the abstract type as their
--      return value
--
procedure CommPort ( )
    Returns (CommPort thePort)
    Signals (OutOfMemory)


--
-- String Procedure
--
-- By convention, all ADTs contain a string procedure which returns some
-- string corresponding to the ADT.  The string can reflect something
-- unique about the instance or simply the name of the ADT. Also by
-- convention, the string procedure is the same as the Utility Vector
-- Display procedure (ADTUdisplay) since they have the same signiture.
--
procedure String (CommPort thePort)
    Returns (string theString)
    Signals (OutOfMemory)


------------------------------------------------------------------
-- Attributes:
------------------------------------------------------------------

--
-- Set Attributes have the following characteristics:
-- 1) The first input argument is always the ADT instance
-- 2) Succeeding input arguments contain the value to be set
-- 3) They only return the instance.
--
procedure BaudRate ( CommPort thePort, integer theRate)
    Returns (thePort)
    Signals (InvalidBaudRate)

--
-- Get Attributes have the following characteristics:
-- 1) The first input argument is always the ADT instance
-- 2) The return arguments are only the attributes being retrieved (ie.
--      never the ADT instance).
--
procedure BaudRate (Commport thePort)
    Returns (integer theRate)

procedure Port ( CommPort thePort, PortType theNewPort)
    Returns (thePort)
    Signals (InvalidPort)

procedure Port (Commport thePort)
    Returns (PortType theCurrentPort)

procedure DataBits ( CommPort thePort, DataBitType theBits)
    Returns (thePort)
    Signals (InvalidDataBits)

procedure DataBits (Commport thePort)
    Returns (DataBitType theBits)

procedure StopBits ( CommPort thePort, StopBitType theBits)
    Returns (thePort)
    Signals (InvalidStopBits)

procedure StopBits (Commport thePort)
    Returns (StopBitType theBits)

procedure Parity ( CommPort thePort, ParityType theParity)
    Returns (thePort)

procedure Parity (Commport thePort)
    Returns (ParityType theParity)


------------------------------------------------------------------
-- Procedures:
------------------------------------------------------------------
procedure Open (CommPort thePort)
    Signals ( PortInUse )

procedure Close (CommPort thePort)


--
-- Select
--
-- Select is a special procedure which can be used as an "array-like"
-- operator.  Select retrieves data from the ADT instance. Select has the
-- following characteristics:
-- 1) The first input argument is always the ADT instance.
-- 2) The second and successive input arguments can be used to identify the
--      item being returned.
-- 3) Select only has one return value, the item being returned.
--
procedure Select (CommPort thePort)
    Returns (string theString)
    Signals (PortNotOpen, OutOfMemory)

--
-- Replace
--
-- Replace is a special procedure which can be used as an "array-like"
-- operator.  Replace is used to set data in the ADT instance. Replace
-- has the following characteristics:
-- 1) Replace always takes the ADT instance as its first input argument.
-- 2) The last input argument should be the item (string, integer, etc) being
--      stored.
-- 3) Replace only returns the ADT instance.
--
procedure Replace (CommPort thePort, string theString)
    Returns (thePort)
    Signals (PortNotOpen)

--
-- EQ procedure
--
-- The EQ procedure is the synonym for the equality operator for an abstract
-- data type.  Its implementation is identical to the implementation of the
-- Utility Vector compare function (ADTUcompare).  The EQ procedure has the
-- following characteristics:
-- 1) It always takes two ADT instances as input
-- 2) It always returns a boolean which reflects the equality of the two
--      input instances.
--
procedure EQ (CommPort portOne, portTwo)
    Returns (Boolean isEqual)



end Module CommPort
------------------------------------------------------------------
-- End of Module
------------------------------------------------------------------
