--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  filedem.pl
--
--**************************************************************************

use File
use Range      -- to create a Range object in Main
use Register   -- to register @functions
use Wkspace    -- for recalculation
constant Comma    = ","
constant Quote    = '"'
constant FileName = "FILEDEM.TXT"
constant ModName  = "Module FILEDEM error: "
procedure Initialize is
-- register the @functions procedures
   begin    -- of signal block
      Register ("Exists",     AtFunction)
      Register ("FileSize",   AtFunction)
      Register ("FileDate",   AtFunction)
   end      -- signal block
      except
         on others do
            Print (ModName || "<<< @Function Register Failure >>>")
      end except
end procedure Initialize
procedure Exists (string FileName) returns (integer Val) is
-- @function that returns 1 if file is found and 0 if not found
   begin  -- signal block
      -- declare File object
      File [FileName] TheFile
      -- check Exists attribute
      if TheFile.Exists = true then
         -- file is found
         return (1)
      else
         -- no file is found
         return (0)
      end if
   end  -- signal block
      except on others do
         -- on signal return "no file found"
         return (1)
      end except
end procedure Exists
procedure FileSize (string Filename) returns (integer Size)
   signals (NA,ERR) is
-- @function that returns the size of a file
   begin  -- signal block
      -- declare File object
      File [FileName] TheFile
      Size = TheFile.Size
   end  -- signal block
      except
         on FileNotFound do
            signal NA
         on others do
         -- all other signals return ERR
            signal ERR
      end except
end procedure FileSize
procedure FileDate (string FileName) returns (string Stamp)
   signals (NA,ERR) is
-- @function that returns the date and timestamp of a file
   begin  -- signal block
      -- declare File ADT
      File [FileName] TheFile
      if TheFile.Exists then
         -- get file date and timestamp
         string d,t = TheFile.LastWrite
         Stamp = d || " " || t
      else
         signal NA
      end if
   end  -- signal block
        -- all signals return ERR
      except on others do
            signal ERR
      end except
end procedure FileDate
procedure Main is
-- prompts the user to highlight a range, and saves the
-- values in the range in a comma delimited text file
   begin  -- signal block
      -- declare a range and prompt the user
      Range [] TheRange
      TheRange = Input ("Select a range to save as ASCII: ",
                         true, TheRange)
      -- get the total number of columns
      integer Cols = TheRange.Columns
      -- get the total number of rows
      integer Rows = TheRange.Rows
      -- declare an output file
      File [FileName, Append] Output
      -- declare a string variable for output
      string Line
      for integer r = 1 to Rows loop
         -- initialize Line to null
         Line = ""
         for integer c = 1 to Cols loop
            case TheRange.DataType (c, r)
               of StringData:
                  -- enclose the line in quotation marks
                  Line = Line || '"' || TheRange [c, r] || '"'
               of NumberData:
               -- convert number to a string
               float Val = TheRange [c, r]
                  Line = Line || string (Val)
               of ERR_Data:
                  Line = Line || "ERR"
               of NA_Data:
                  Line = Line || "NA"
            end case
            -- if there are more cells to check
            if c <> Cols then
               -- separate the values with a comma
               Line = Line || ','
            end if
         end loop
         -- write line to file
         Write (OutPut, Line)
            except on others do
               -- go to the next iteration
               continue
            end except
      end loop
      -- close the file
      Close (OutPut)
      Calc (Workspace)
   end  -- signal block
      except on others do
         -- do nothing on any signal
      end except
end procedure Main
procedure Terminate is
-- remove the used file FILEDEM.TXT
   begin  -- signal block
      File [FileName] Output
      Delete (Output)
   end  -- signal block
      except on others do
         Print (ModName || "<<< Remove Failure >>>")
      end except
end procedure Terminate

end module FileDem
