--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  median.pl
--
--**************************************************************************

module median is

use Range              -- the module that contains the Range ADT and procedures
use Register   -- the module that contains the Register procedure

-- GetSize determines the size of the input range
-- Inputs: a Range object
-- Returns: an integer

procedure GetSize (Range thisRange) returns (integer Size) is

       Size = thisRange.Sheets * thisRange.Rows * thisRange.Columns
    except on others (string SignalName) do
      Print("Signal ", SignalName, " raised.")
      end except

       end procedure GetSize

-- SortList copies the values in the range into a one-dimensional
-- array (the List) and then sorts them.
-- Inputs: a Range object and an integer (Size)
-- Returns: the sorted array

procedure SortList (Range thisRange, integer Size)
       returns (float array [Size] theList) is

 begin
       integer i = 0
       for integer s = 1 to thisRange.Sheets loop
               for integer c = 1 to thisRange.Columns loop
                       for integer r = 1 to thisRange.Rows loop
                               i = i + 1
                               theList[i] = thisRange[s, c, r]
                       end loop
               end loop
       end loop

       loop
               boolean SortedItem = false
               for i = 1 to Size - 1 loop
                       if theList[i] > theList[i+1] then
                               float temp = theList[i]
                               theList[i] = theList[i+1]
                               theList[i+1] = temp
                               SortedItem = true
                       end if
               end loop
               exit when NOT SortedItem
       end loop
 end
  except on others (string SignalName) do
    Print("Signal ", SignalName, " raised.")
    end except

       end procedure SortList

-- Median calculates the median of a set of values
-- Inputs: a Range object
-- Returns: a float

procedure Median (Range thisRange) returns (float thisMedian) is

 begin
       integer Size = GetSize(thisRange)
       float array [Size] theList
       theList = SortList(thisRange, Size)

       integer Middle = Integer(Size / 2)
       if Even(Size) then
               return((theList[Middle] + theList[Middle+1]) / 2)
       else
               return(theList[Middle + 1])
       end if
 end
  except on others (string SignalName) do
    Print("Signal ", SignalName, " raised.")
    end except

       end procedure Median

procedure Initialize is

 begin
       Register("Median", AtFunction)
               except on Others do  --handle signal
                       Print("Unable to register @MEDIAN")
                       end except
 end
  except on others (string SignalName) do
    Print("Signal ", SignalName, " raised.")
    end except

       end procedure Initialize
